#!/bin/bash
# BashPodder (BP4iP)
# Copyright 2005 Justin Gerace

# This uses a small amount of BashPodder code, copyright 2004 Linc
# http://linc.homeunix.org:8080/scripts/bashpodder/

# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License (see "LICENSE" for details)

# Set stuff
podcount=$'0'
poddir=$HOME'/Podcasts'

if test ! -d $poddir
then
	mkdir $poddir
fi

# Create the Main Menu
function mainmenu()
{
Xdialog --title "BashPodder GUI" \
        --cancel-label "Exit" \
	--item-help \
	--icon BPGUI.xpm \
	--menu "" 400x350 0 \
	"Scan for New Podcasts" "" "Scan Podcasts and download new ones" \
	"Play Podcasts" "" "Play a Podcast that you downloaded" \
	"Add a Podcast" "" "Use this to add a new Podcast" \
	"Modify Podcasts" "" "Use this to modify your existing Podcasts" \
	"Remove a Podcast" "" "Use this to remove an existing Podcast" 2> /tmp/bp4ip.tmp.$$
choice=`cat /tmp/bp4ip.tmp.$$`
rm -f /tmp/bp4ip.tmp.$$

case "$choice" in
"Scan for New Podcasts")
	podscan
	mainmenu
;;
"Play Podcasts")
	play
	mainmenu
;;
"Add a Podcast")
	addpodcast
	mainmenu
;;
"Modify Podcasts")
	editpodcasts
	mainmenu
;;
"Remove a Podcast")
	removepodcast
	mainmenu
;;
esac
exit 0
}

# Scan for Podcasts
function podscan()
{
if `Xdialog --title "BashPodder GUI" \
	    --ok-label "Continue >" \
	    --cancel-label "Main Menu" \
	    --yesno "Ready to scan for Podcasts" 6 45`
then
	rm -f temp.log
	while read podcast
	do		
		podcastaddress=`expr index "$podcast" @`
		podcastURL=${podcast:$podcastaddress}
		datadir=${podcast:0:$podcastaddress-1}
		podcastdir=$poddir'/'$datadir
		wget -q $podcastURL -O /tmp/bpgui.tmp.$$
		title=$(cat /tmp/bpgui.tmp.$$ | tr '\r' '\n' | tr \' \" | sed -n 's/.*<title>\([^">]*\)<\/title>.*/\1/p' | head -n1)
		Xdialog --title "BashPodder GUI" --no-buttons --interval 2 --infobox "Now Scanning:\n$title" 6 45 &
		file=$(cat /tmp/bpgui.tmp.$$ | tr '\r' '\n' | tr \' \" | sed -n 's/.*url="\([^"]*\)".*/\1/p')
	#	rm /tmp/bpgui.tmp.$$
		for url in $file
		do
			echo $url >> temp.log
			if ! grep "$url" podcast.log > /dev/null
			then
				if test ! -d $podcastdir
				then
					Xdialog --title "Notice" --no-buttons --interval 10 --infobox "Making $title Podcast\ndirectory..." 6 45
					mkdir $podcastdir
				fi
				# Get download size
				progress="0"
				i="1"
				newurl=$url
				while [ $i -ne 0 ]
				do
					i=`expr index "$newurl" \/`
					newurl=${newurl:$i}
				done
				if test ! -e $podcastdir'/'$newurl
				then
				{
					wget -v -P $podcastdir "$url" 2> .download.status &
					echo 0
					dlsize=$(cat /tmp/bpgui.tmp.$$ | tr '\r' '\n' | tr \' \" | sed -n 's/.*'$newurl'"\ length="\([^"]*\)".*/\1/p')
					while [ "$progress" != "$dlsize" ]
					do
						progress="`ls -l "$podcastdir/$newurl" |sed -e \"s/\ \+/,/g\"|cut -d\, -f5`"
						echo $[progress*100/dlsize]
						sleep 1s
					done
					echo 100
				}  | Xdialog --title "Downloading" --gauge "Downloading\n$url" 7 60 0
				rm .download.status

				Xdialog --title "Notice" --no-buttons --interval 10 --infobox "$url\nRecieved" 7 60
				url=$newurl
				let "podcount += 1"
				fi
			fi
			ls -1 --time=ctime $podcastdir | cat > $podcastdir/$datadir.m3u
			grep -v $datadir $podcastdir/$datadir.m3u | cat > $podcastdir/$datadir.m3u
		done
	done < bp.conf
	if [ $podcount = '1' ]
	then
		podcast=$'Podcast'
		were=$'was'
	else
		podcast=$'Podcasts'
		were=$'were'
	fi
	total=$podcount
	if [ $podcount = '0' ]
	then
		total="No"
	fi
	Xdialog --title "Notice" \
		--ok-label "Main Menu" \
		--msgbox "$total $podcast $were downloaded to your computer" 6 45
		cat podcast.log >> temp.log
		sort temp.log | uniq > podcast.log
		rm temp.log
		return 0
fi
return 0
}

# Add a podcast
function addpodcast()
{

Xdialog --title "BashPodder GUI" \
	--ok-label "Next >" \
	--no-cancel \
	--inputbox "What is the Podcast feed URL?" 8 45 2> /tmp/bp4ip.tmp.$$
feedurl=`cat /tmp/bp4ip.tmp.$$`
rm -f /tmp/bp4ip.tmp.$$

ldir=$(wget -q $feedurl -O - | tr '\r' '\n' | tr \' \" | sed -n 's/.*<title>\([^">]*\)<\/title>.*/\1/p' | head -n1)
i="1"
while [ $i -ne 0 ]
do
	i=`expr index "$ldir" " "`
	if [ $i -ne 0 ]
	then
		word=${ldir:0:$i-1}
		ldir=$word${ldir:$i}
	fi
done

Xdialog --title "BashPodder GUI" \
	--ok-label "Next >" \
	--no-cancel \
	--inputbox "What should the Podcast name be?\n(this will be the directory it is stored in)" 8 45 "$ldir" 2> /tmp/bp4ip.tmp.$$
ldir=`cat /tmp/bp4ip.tmp.$$`
rm -f /tmp/bp4ip.tmp.$$
if `Xdialog --title "BashPodder GUI" \
	--ok-label "Yes" \
	--cancel-label "No" \
	--yesno "Is this information correct?\n\nLocal Directory: '$ldir'\n$feedurl" 9 69`
then
	echo $ldir'@'$feedurl >> bp.conf
	title=$(wget -q $feedurl -O - | tr '\r' '\n' | tr \' \" | sed -n 's/.*<title>\([^">]*\)<\/title>.*/\1/p' | head -n1)
	Xdialog --title "Notice" \
		--msgbox "'$title'\nadded successfully" 7 50
else
	Xdialog --title "Notice" \
		--ok-label "Main Menu" \
		--msgbox "Nothing added" 6 45
fi
return 0
}

# Edit Podcasts
function editpodcasts()
{
echo 'Xdialog --title "BashPodder GUI" --cancel-label "Main Menu" --item-help --icon BPGUI.xpm --menu "" 400x350 0 \' > .pselect.mnu
podcastnumber=$'0'
podcasttotal=$'0'
while read podcast
do
	let "podcasttotal += 1"
done < bp.conf
while read podcast
do
	let "podcastnumber += 1"
	podcastaddress=`expr index "$podcast" @`
	feedurl=${podcast:$podcastaddress}
	ldir=${podcast:0:$podcastaddress-1}
	if [ $podcastnumber = $podcasttotal ]
	then
		echo '"'$ldir'" "" "Modify '$ldir'" 2> /tmp/bp4ip.tmp.$$' >> .pselect.mnu
	else
		echo '"'$ldir'" "" "Modify '$ldir'" \' >> .pselect.mnu
	fi
done < bp.conf
source .pselect.mnu
pselection=`cat /tmp/bp4ip.tmp.$$`
rm /tmp/bp4ip.tmp.$$
if [ "$pselection" = "" ]
then
	return 0
fi
pselection=`grep "$pselection" bp.conf`
podcastaddress=`expr index "$pselection" @`
podcastURL=${pselection:$podcastaddress}
ldir=${pselection:0:$podcastaddress-1}

Xdialog --title "BashPodder GUI" \
	--no-cancel \
	--2inputsbox "Change to what?" 15 60 \
	"Local Directory:" "$ldir" \
	"URL:" "$podcastURL" 2> /tmp/bp4ip.tmp.$$
newpodcast=`cat /tmp/bp4ip.tmp.$$`
rm /tmp/bp4ip.tmp.$$
oldpodcast=$ldir'/'$podcastURL
if [ "$newpodcast" = "$oldpodcast" ]
then
	Xdialog --title "Notice" --msgbox "Nothing Changed" 6 45
	editpodcasts
	return 0
fi
slash=`expr index "$newpodcast" /`
newldir=${newpodcast:0:$slash-1}
newurl=${newpodcast:$slash}
grep -v "$ldir" bp.conf | cat > bp.conf
echo $newldir'@'$newpodcast >> bp.conf

Xdialog --title "Notice" \
	--msgbox "'$newldir' Changed" 6 45
editpodcasts
return 0
}

# Remove a Podcast
function removepodcast()
{
echo 'Xdialog --title "BashPodder GUI" --cancel-label "Main Menu" --ok-label "Remove" --item-help --icon BPGUI.xpm --menu "" 400x350 0 \' > .pselect.mnu
podcastnumber=$'0'
podcasttotal=$'0'
while read podcast
do
	let "podcasttotal += 1"
done < bp.conf
while read podcast
do
	let "podcastnumber += 1"
	podcastaddress=`expr index "$podcast" @`
	ldir=${podcast:0:$podcastaddress-1}
	if [ $podcastnumber = $podcasttotal ]
	then
		echo '"'$ldir'" "" "Remove '$ldir'" 2> /tmp/bp4ip.tmp.$$' >> .pselect.mnu
	else
		echo '"'$ldir'" "" "Remove '$ldir'" \' >> .pselect.mnu
	fi
done < bp.conf
source .pselect.mnu
pselection=`cat /tmp/bp4ip.tmp.$$`
rm /tmp/bp4ip.tmp.$$
if [ "$pselection" = "" ]
then
	return 0
fi
if `Xdialog --title "BashPodder GUI" --ok-label "Remove" --cancel-label "Keep" --yesno "Really remove $pselection?" 6 45`
then
	grep -v "$pselection" bp.conf > bp.conf.new
	rm bp.conf
	mv bp.conf.new bp.conf
	Xdialog --title "Notice" \
		--ok-label "Main Menu" \
		--msgbox "$pselection Removed" 6 45
	return 0
fi
removepodcast
return 0
}

# Play a podcast
function play()
{
echo 'Xdialog --title "BashPodder GUI" --cancel-label "Main Menu" --item-help --icon BPGUI.xpm --menu "" 400x350 0 \' > .pselect.mnu
podcastnumber=$'0'
podcasttotal=$'0'
while read podcast
do
	let "podcasttotal += 1"
done < bp.conf
while read podcast
do
	let "podcastnumber += 1"
	podcastaddress=`expr index "$podcast" @`
	feedurl=${podcast:$podcastaddress}
	ldir=${podcast:0:$podcastaddress-1}
	if [ $podcastnumber = $podcasttotal ]
	then
		echo '"'$ldir'" "" "Play Podcasts from: '$ldir'" 2> /tmp/bp4ip.tmp.$$' >> .pselect.mnu
	else
		echo '"'$ldir'" "" "Play podcasts from: '$ldir'" \' >> .pselect.mnu
	fi
done < bp.conf
source .pselect.mnu
pselection=`cat /tmp/bp4ip.tmp.$$`
rm /tmp/bp4ip.tmp.$$
if [ "$pselection" = "" ]
then
	return 0
fi
xmms $poddir/$pselection/$pselection.m3u &
return 0
}

# Convert old bp.conf into new bp.conf
function convert()
{
rm -f bp.conf.new
if ! `Xdialog --title "Notice" \
	--ok-label "Update >" \
	--cancel-label "Exit" \
	--yesno "It appears that your bp.conf needs to be updated.\nProceed with the update?" 7 47`
then
	exit 0
fi
confcount="0"
conftotal="0"
while read oldconf
do
	let "conftotal += 1"
done < bp.conf
{
echo 0
while read oldconf
do
	let "confcount += 1"
	title=$(wget -q $oldconf -O - | tr '\r' '\n' | tr \' \" | sed -n 's/.*<title>\([^">]*\)<\/title>.*/\1/p' | head -n1)
	echo "XXX"
	echo "Now updating $title"
	echo "XXX"
	echo $[confcount*100/conftotal]
	i="1"
	while [ $i -ne 0 ]
	do
		i=`expr index "$title" " "`
		if [ $i -ne 0 ]
		then
			word=${title:0:$i-1}
			title=$word${title:$i}
		fi
	done
	echo $title'@'$oldconf >> bp.conf.new
done < bp.conf
sleep 1s
} | Xdialog --title "Updating" --gauge "Preparing to update..." 7 60
Xdialog --title "Notice" \
	--ok-label "Main Menu" \
	--msgbox "Update complete.\nYou should further modify the Podcasts" 7 45
mv bp.conf bp.conf.old
mv bp.conf.new bp.conf
#Xdialog --title "Notice" \
#	--ok-label "Main Menu" \
#	--msgbox "This feature is currently under development" 6 45
return 0
}

# Display welcome message
Xdialog --title "BashPodder GUI" \
	--no-buttons \
	--interval 2.5 \
	--infobox "Welcome to BashPodder" 6 45

# Check if bp.conf needs to be updated
grepconf=`grep @ bp.conf`
if [ "$grepconf" = "" ]
then
	if test -e bp.conf
	then
		convert
	fi
fi

# Launch Main Menu
mainmenu
